@@ -65,8 +65,10 @@ enum {
         SECRET_INFO_QUERY,
         READY,
         RESET,
+        AUTHENTICATION_FAILED,
         SELECTED_USER_CHANGED,
         DEFAULT_LANGUAGE_NAME_CHANGED,
+        DEFAULT_LAYOUT_NAME_CHANGED,
         DEFAULT_SESSION_NAME_CHANGED,
         TIMED_LOGIN_REQUESTED,
         USER_AUTHORIZED,
@@ -174,6 +176,13 @@ on_default_language_name_changed (GdmGreeterClient *client,
         emit_string_signal_for_message (client, "DefaultLanguageNameChanged", message, DEFAULT_LANGUAGE_NAME_CHANGED);
 }
 
+static void
+on_default_layout_name_changed (GdmGreeterClient *client,
+                                DBusMessage      *message)
+{
+        emit_string_signal_for_message (client, "DefaultLayoutNameChanged", message, DEFAULT_LAYOUT_NAME_CHANGED);
+}
+
 static void
 on_default_session_name_changed (GdmGreeterClient *client,
                                  DBusMessage      *message)
@@ -247,6 +256,17 @@ on_reset (GdmGreeterClient *client,
                        0);
 }
 
+static void
+on_authentication_failed (GdmGreeterClient *client,
+                          DBusMessage      *message)
+{
+        g_debug ("GdmGreeterClient: Authentication failed");
+
+        g_signal_emit (client,
+                       gdm_greeter_client_signals[AUTHENTICATION_FAILED],
+                       0);
+}
+
 static gboolean
 send_dbus_string_method (DBusConnection *connection,
                          const char     *method,
@@ -451,6 +471,14 @@ gdm_greeter_client_call_select_language (GdmGreeterClient *client,
                                  text);
 }
 
+void
+gdm_greeter_client_call_select_layout (GdmGreeterClient *client,
+                                       const char       *text)
+{
+        send_dbus_string_method (client->priv->connection,
+                                 "SelectLayout",
+                                 text);
+}
 void
 gdm_greeter_client_call_select_user (GdmGreeterClient *client,
                                      const char       *text)
@@ -629,10 +657,14 @@ client_dbus_handle_message (DBusConnection *connection,
                 on_ready (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "Reset")) {
                 on_reset (client, message);
+        } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "AuthenticationFailed")) {
+                on_authentication_failed (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "SelectedUserChanged")) {
                 on_selected_user_changed (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "DefaultLanguageNameChanged")) {
                 on_default_language_name_changed (client, message);
+        } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "DefaultLayoutNameChanged")) {
+                on_default_layout_name_changed (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "DefaultSessionNameChanged")) {
                 on_default_session_name_changed (client, message);
         } else if (dbus_message_is_signal (message, GREETER_SERVER_DBUS_INTERFACE, "TimedLoginRequested")) {
@@ -712,7 +744,7 @@ gdm_greeter_client_start (GdmGreeterClient *client,
                         g_set_error (error,
                                      GDM_GREETER_CLIENT_ERROR,
                                      GDM_GREETER_CLIENT_ERROR_GENERIC,
-                                     local_error.message);
+                                     "%s", local_error.message);
                         dbus_error_free (&local_error);
                 } else {
                         g_warning ("Unable to open connection");
@@ -875,6 +907,16 @@ gdm_greeter_client_class_init (GdmGreeterClientClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
+        gdm_greeter_client_signals[AUTHENTICATION_FAILED] =
+                g_signal_new ("authentication-failed",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterClientClass, authentication_failed),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
         gdm_greeter_client_signals[SELECTED_USER_CHANGED] =
                 g_signal_new ("selected-user-changed",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -895,6 +937,16 @@ gdm_greeter_client_class_init (GdmGreeterClientClass *klass)
                               g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
                               1, G_TYPE_STRING);
+        gdm_greeter_client_signals[DEFAULT_LAYOUT_NAME_CHANGED] =
+                g_signal_new ("default-layout-name-changed",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterClientClass, default_layout_name_changed),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1, G_TYPE_STRING);
         gdm_greeter_client_signals[DEFAULT_SESSION_NAME_CHANGED] =
                 g_signal_new ("default-session-name-changed",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -910,7 +962,7 @@ gdm_greeter_client_class_init (GdmGreeterClientClass *klass)
                 g_signal_new ("timed-login-requested",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (GdmGreeterClientClass, default_session_name_changed),
+                              G_STRUCT_OFFSET (GdmGreeterClientClass, timed_login_requested),
                               NULL,
                               NULL,
                               gdm_marshal_VOID__STRING_INT,
